package org.convey;

import java.util.*;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

/**
 * GraphicPanel is a JPanel implementation that manages the Graphic objects.
 * This class provides the link between the Graphic objects and Java Swing's
 * painting and input mechanisms.
 *
 * @author Andy Ames
 * @version 0.1
 */
public class GraphicPanel extends JPanel {

	protected GraphicFrame graphicFrame = null;

	/**
	 * Vector of all Graphic objects contained in the GraphicPanel. The Graphic
	 * objects are ordered from back to front.
	 *
	 * @associates <{org.convey.Graphic}>
	 * @supplierCardinality 0..*
	 */
	protected Vector graphics = new Vector();

	protected Builder builder = null;

	/**
	 * Default constructor. We're using white for the default background color.
	 * Then, we're adding several test ShapeGraphic instances.
	 */
  public GraphicPanel(GraphicFrame graphicFrame) {
		super.setBackground(Color.white);

		this.graphicFrame = graphicFrame;
  }

	/**
	 * Paints the GraphicPanel component be painting all Graphic objects.
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);

		Graphics2D g2d = (Graphics2D)g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
											   RenderingHints.VALUE_ANTIALIAS_ON);

		Iterator iter = this.graphics.iterator();
		while(iter.hasNext()) {
			Graphic graphic = (Graphic)iter.next();
			graphic.paint(g2d);
		}

		if(this.builder != null)
			this.builder.paint(g2d);
	}

	/**
	 * Brings the specified Graphic object to the front by placing it at the
	 * end of the graphics list.
	 */
	public void bringToFront(Graphic graphic) {
		this.graphics.remove(graphic);
		this.graphics.add(graphic);
	}

	public void addGraphic(Graphic graphic) {
		this.graphics.add(graphic);
	}

	/**
	 * Returns the Graphic instance count.
	 */
	public int graphicCount() {
		return this.graphics.size();
	}

	/**
	 * Returns the Graphic instance at the specified index.
	 */
	public Graphic getGraphic(int index) {
		return (Graphic)this.graphics.elementAt(index);
	}

	public Iterator graphicIterator() {
		return this.graphics.iterator();
	}

	public ListIterator graphicListIterator() {
		return this.graphics.listIterator();
	}

	public ListIterator graphicListIterator(int index) {
		return this.graphics.listIterator(index);
	}

	public void setBuilder(Builder builder) {
		if(this.builder != builder) {
			if(this.builder != null) {
				super.removeMouseListener(this.builder);
				super.removeMouseMotionListener(this.builder);
				super.removeKeyListener(this.builder);
			}

			Graphic selected = null;
			if(this.builder instanceof SelectionBuilder)
				selected = ((SelectionBuilder)this.builder).getSelected();

			if(selected != null)
				selected.setSelected(false);

			if(builder != null && this.builder != null &&
				 builder instanceof SelectionBuilder &&
				 this.builder instanceof SelectionBuilder)
				((SelectionBuilder)builder).setSelected(selected);

			this.builder = builder;
			if(this.builder != null) {
				super.addMouseListener(this.builder);
	  		super.addMouseMotionListener(this.builder);
				super.addKeyListener(this.builder);
			}

			super.repaint();
		}
	}

	public boolean isFocusTraversable() {
	  return true;
	}

}
