package org.convey;

import java.util.*;

import java.awt.*;
import java.awt.geom.*;

/**
 * ShapeGraphic is a Graphic implementation for constructing and manipulating
 * arbitrary shapes.
 *
 * @author Andy Ames
 * @version 0.1
 */
public class ShapeGraphic extends Graphic {

	public static final int SEG_CLOSE = 0;

	public static final int SEG_MOVETO = 1;

	public static final int SEG_POINT = 2;

	public static final int SEG_LINETO = 3;

	public static final int SEG_QUADTO = 4;

	public static final int SEG_CUBICTO = 5;

	private static final int MAX_SEG = 5;

	/**
	 * Vector of coordinates for the path of this ShapeGraphic. Each element of
	 * the Vector is an array of six ints (three coordinate pairs). The
	 * interpretation of these coordinate pairs depends on the corresponding
	 * coordinate type in the typeVector.
	 */
	protected Vector coordVector = new Vector();

	/**
	 * Vector of coordinat types for the path of this ShapeGraphic. Each element
	 * of the Vector is an Integer containing the type value for the corresponding
	 * coordinates in coordVector. The possible types are SEG_CLOSE, SEG_CUBICTO,
	 * SEG_POINT, SEG_LINETO, SEG_MOVETO, and SEG_QUADTO.
	 */
	protected Vector typeVector = new Vector();

	/**
	 * Constructor.
	 */
	public ShapeGraphic(GraphicPanel graphicPanel) {
		super(graphicPanel);
	}

	/**
	 * Paints the ShapeGraphic object onto the Graphics2D object.
	 */
	public void paint(Graphics2D g2d) {
		if(g2d != null && this.typeVector.size() > 0) {
			g2d.setStroke(new BasicStroke());
			g2d.setColor(Color.black);

			Iterator typeIter = this.typeVector.iterator();
			Iterator coordIter = this.coordVector.iterator();
			Point lastMoveTo = new Point(0, 0);
			Point current = new Point(0, 0);
			while(typeIter.hasNext()) {
				int type = ((Integer)typeIter.next()).intValue();
				int[] coords = (int[])coordIter.next();
				switch(type) {
					case SEG_POINT:
						g2d.fillOval(lastMoveTo.x - 1, lastMoveTo.y - 1, 2, 2);
					case SEG_CLOSE:
						g2d.drawLine(current.x, current.y, lastMoveTo.x, lastMoveTo.y);
						current.x = lastMoveTo.x;
						current.y = lastMoveTo.y;
						break;
					case SEG_CUBICTO:
						CubicCurve2D cubic = new CubicCurve2D.Float(current.x, current.y,
																		coords[0], coords[1],
																	coords[2], coords[3],
																	coords[4], coords[5]);
						g2d.draw(cubic);
						current.x = coords[4];
						current.y = coords[5];
						break;
					case SEG_LINETO:
						g2d.drawLine(current.x, current.y, coords[0], coords[1]);
						current.x = coords[0];
						current.y = coords[1];
						break;
					case SEG_MOVETO:
						current.x = lastMoveTo.x = coords[0];
						current.y = lastMoveTo.y = coords[1];
						break;
					case SEG_QUADTO:
						QuadCurve2D quad = new QuadCurve2D.Float(current.x, current.y,
																										 coords[0], coords[1],
																										 coords[2], coords[3]);
						g2d.draw(quad);
						current.x = coords[2];
						current.y = coords[3];
						break;
				}
			}
		}
	}

	public void paintSelection(Graphics2D g2d) {
		if(g2d != null && this.typeVector.size() > 0) {
			float[] dash = { 6, 6 };
			BasicStroke selStroke = new BasicStroke(1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0.0f, dash, 0.0f);
			g2d.setStroke(selStroke);
			g2d.setColor(Color.white);

			Iterator typeIter = this.typeVector.iterator();
			Iterator coordIter = this.coordVector.iterator();
			Point lastMoveTo = new Point(0, 0);
			Point current = new Point(0, 0);
			while(typeIter.hasNext()) {
				int type = ((Integer)typeIter.next()).intValue();
				int[] coords = (int[])coordIter.next();
				switch(type) {
					case SEG_POINT:
						g2d.fillOval(lastMoveTo.x - 1, lastMoveTo.y - 1, 2, 2);
					case SEG_CLOSE:
						g2d.drawLine(current.x, current.y, lastMoveTo.x, lastMoveTo.y);
						current.x = lastMoveTo.x;
						current.y = lastMoveTo.y;
						break;
					case SEG_CUBICTO:
						CubicCurve2D cubic = new CubicCurve2D.Float(current.x, current.y,
						coords[0], coords[1],
						coords[2], coords[3],
																												coords[4], coords[5]);
						g2d.draw(cubic);
						current.x = coords[4];
						current.y = coords[5];
						break;
					case SEG_LINETO:
						g2d.drawLine(current.x, current.y, coords[0], coords[1]);
						current.x = coords[0];
						current.y = coords[1];
						break;
					case SEG_MOVETO:
						current.x = lastMoveTo.x = coords[0];
						current.y = lastMoveTo.y = coords[1];
						break;
					case SEG_QUADTO:
						QuadCurve2D quad = new QuadCurve2D.Float(current.x, current.y,
																	coords[0], coords[1],
																	coords[2], coords[3]);
						g2d.draw(quad);
						current.x = coords[2];
						current.y = coords[3];
						break;
				}
			}

			selStroke = new BasicStroke(1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0.0f, dash, 6.0f);
			g2d.setStroke(selStroke);
			g2d.setColor(Color.gray);

			typeIter = this.typeVector.iterator();
			coordIter = this.coordVector.iterator();
			lastMoveTo = new Point(0, 0);
			current = new Point(0, 0);
			while(typeIter.hasNext()) {
				int type = ((Integer)typeIter.next()).intValue();
				int[] coords = (int[])coordIter.next();
				switch(type) {
					case SEG_POINT:
						g2d.fillOval(lastMoveTo.x - 1, lastMoveTo.y - 1, 2, 2);
					case SEG_CLOSE:
						g2d.drawLine(current.x, current.y, lastMoveTo.x, lastMoveTo.y);
						current.x = lastMoveTo.x;
						current.y = lastMoveTo.y;
						break;
					case SEG_CUBICTO:
						CubicCurve2D cubic = new CubicCurve2D.Float(current.x, current.y,
																												coords[0], coords[1],
																												coords[2], coords[3],
																												coords[4], coords[5]);
						g2d.draw(cubic);
						current.x = coords[4];
						current.y = coords[5];
						break;
					case SEG_LINETO:
						g2d.drawLine(current.x, current.y, coords[0], coords[1]);
						current.x = coords[0];
						current.y = coords[1];
						break;
					case SEG_MOVETO:
						current.x = lastMoveTo.x = coords[0];
						current.y = lastMoveTo.y = coords[1];
						break;
					case SEG_QUADTO:
						QuadCurve2D quad = new QuadCurve2D.Float(current.x, current.y,
																										 coords[0], coords[1],
																										 coords[2], coords[3]);
						g2d.draw(quad);
						current.x = coords[2];
						current.y = coords[3];
						break;
				}
			}
		}
	}

	public boolean contains(Point pt) {
		BasicStroke hotspot = new BasicStroke(6);

		Iterator typeIter = this.typeVector.iterator();
		Iterator coordIter = this.coordVector.iterator();
		Point lastMoveTo = new Point(0, 0);
		Point current = new Point(0, 0);
		while(typeIter.hasNext()) {
			int type = ((Integer)typeIter.next()).intValue();
			int[] coords = (int[])coordIter.next();
			switch(type) {
				case SEG_POINT:
					Ellipse2D point = new Ellipse2D.Float(lastMoveTo.x - 1,
																								lastMoveTo.y - 1, 2, 2);
				  if(point.contains(pt))
						return true;
					break;
				case SEG_CLOSE:
					Line2D close = new Line2D.Float(current.x, current.y,
																				  lastMoveTo.x, lastMoveTo.y);
				  if(hotspot.createStrokedShape(close).contains(pt))
						return true;
					current.x = lastMoveTo.x;
					current.y = lastMoveTo.y;
					break;
				case SEG_CUBICTO:
					CubicCurve2D cubic = new CubicCurve2D.Float(current.x, current.y,
																										  coords[0], coords[1],
																											coords[2], coords[3],
																											coords[4], coords[5]);
				  if(hotspot.createStrokedShape(cubic).contains(pt))
						return true;
					current.x = coords[4];
					current.y = coords[5];
					break;
				case SEG_LINETO:
					Line2D line = new Line2D.Float(current.x, current.y,
																				 coords[0], coords[1]);
				  if(hotspot.createStrokedShape(line).contains(pt))
						return true;
					current.x = coords[0];
					current.y = coords[1];
					break;
				case SEG_MOVETO:
					lastMoveTo.x = coords[0];
					lastMoveTo.y = coords[1];
					current.x = coords[0];
					current.y = coords[1];
					break;
				case SEG_QUADTO:
					QuadCurve2D quad = new QuadCurve2D.Float(current.x, current.y,
																								   coords[0], coords[1],
																									 coords[2], coords[3]);
				  if(hotspot.createStrokedShape(quad).contains(pt))
						return true;
					current.x = coords[2];
					current.y = coords[3];
					break;
			}
		}

		return false;
	}

	public void translate(int tx, int ty) {
		Iterator typeIter = this.typeVector.iterator();
		Iterator coordIter = this.coordVector.iterator();
		while(typeIter.hasNext()) {
			int type = ((Integer)typeIter.next()).intValue();
			int[] coords = (int[])coordIter.next();
			switch(type) {
				case SEG_POINT:
				case SEG_CLOSE:
					break;
				case SEG_CUBICTO:
					coords[0] += tx;
					coords[1] += ty;
					coords[2] += tx;
					coords[3] += ty;
					coords[4] += tx;
					coords[5] += ty;
					break;
				case SEG_LINETO:
					coords[0] += tx;
					coords[1] += ty;
					break;
				case SEG_MOVETO:
					coords[0] += tx;
					coords[1] += ty;
					break;
				case SEG_QUADTO:
					coords[0] += tx;
					coords[1] += ty;
					coords[2] += tx;
					coords[3] += ty;
					break;
			}
		}
	}

	public int coordCount() {
		return this.typeVector.size();
	}

	/**
	 * Gets the coordinates and corresponding type at a given index into the
	 * coordinate Vector.
	 *
	 * @throw ArrayIndexOutOfBoundsException
	 */
	public int getCoords(int index, int[] coords) {
		System.arraycopy(this.coordVector.elementAt(index), 0, coords, 0, 6);
		return ((Integer)this.typeVector.elementAt(index)).intValue();
	}

	/**
	 * Sets the coordinates and corresponding type at a given index into the
	 * coordinate Vector.
	 *
	 * @throw ArrayIndexOutOfBoundsException
	 */
	public void setCoords(int index, int type, int[] coords) {
		System.arraycopy(coords, 0, this.coordVector.elementAt(index), 0, 6);
		this.typeVector.remove(index);
		this.typeVector.add(index, new Integer(type));
	}

	/**
	 * Removes and returns the coordinates and corresponding type at a given index
	 * into the coordinate Vector.
	 *
	 * @throw ArrayIndexOutOfBoundsException
	 */
	public int removeCoords(int index, int[] coords) {
		System.arraycopy(this.coordVector.elementAt(index), 0, coords, 0, 6);
		int type = ((Integer)this.typeVector.elementAt(index)).intValue();
		this.coordVector.remove(index);
		this.typeVector.remove(index);
		return type;
	}

	/**
	 * Adds a coordinate set and corresponding type at a given index into the
	 * coordinate Vector.
	 *
	 * @throw ArrayIndexOutOfBoundsException
	 */
	public void addCoords(int index, int type, int[] coords) {
		this.coordVector.add(index, coords.clone());
		this.typeVector.add(index, new Integer(type));
	}

}
